package Account

import (
	"errors"
	"fmt"
	"github.com/astaxie/beego/logs"
	"github.com/beego/beego/v2/adapter/orm"
	orm2 "github.com/beego/beego/v2/client/orm"
	userlibs "gogs.baozhida.cn/zoie/ERP_libs/User"
	"gogs.baozhida.cn/zoie/ERP_libs/lib"
	"time"
)

type User struct {
	Id      int    `orm:"column(ID);size(11);auto;pk"`
	T_uuid  string `orm:"size(32);null"`      //
	T_power string `orm:"size(8);default(0)"` // 权限ID
	T_name  string `orm:"size(256);null"`     //
	T_user  string `orm:"size(256);null"`     //
	T_pass  string `orm:"size(256);null"`     //
	T_dept  int    `orm:"size(8);null"`       // 部门
	T_post  int    `orm:"size(8);null"`       // 岗位

	T_dept_leader int `orm:"size(8);default(0)"` // 部门负责人 0-否 1-是

	T_sex                 int    `orm:"size(8);default(1)"`  // 性别 1-男 2-女
	T_id_card             string `orm:"size(32);null"`       // 身份证号
	T_nation              string `orm:"size(32);null"`       // 民族
	T_school              string `orm:"size(32);null"`       // 毕业院校
	T_major               string `orm:"size(32);null"`       // 专业
	T_education           string `orm:"size(32);null"`       // 学历
	T_phone               string `orm:"size(32);null"`       // 联系电话
	T_marry               int    `orm:"size(8);null"`        // 婚否 0-未婚 1-已婚
	T_spouse_name         string `orm:"size(32);null"`       // 配偶姓名
	T_spouse_phone        string `orm:"size(32);null"`       // 联系电话
	T_entry_time          string `orm:"size(32);null"`       // 入职时间
	T_positive_time       string `orm:"size(32);null"`       // 转正时间
	T_entry_type          string `orm:"size(32);default(1)"` // 入职类型 1-全职 2-兼职 3-实习生
	T_contract_start_time string `orm:"size(32);null"`       // 劳动合同开始时间
	T_contract_end_time   string `orm:"size(32);null"`       // 劳动合同结束时间
	T_expire              int    `orm:"size(8);default(0)"`  // 是否到期 0-否 1-是
	T_remark              string `orm:"size(32);null"`       // 备注
	T_cold_uuid           string `orm:"size(32);null"`       // 关联冷链uuid

	T_State    int       `orm:"size(200);default(1)"`                                  //  0删除  1 正常 2 离职
	CreateTime time.Time `orm:"column(create_time);type(timestamp);null;auto_now_add"` //auto_now 每次 model 保存时都会对时间自动更新
	UpdateTime time.Time `orm:"column(update_time);type(timestamp);null;auto_now"`     //auto_now_add 第一次保存时才设置时间
}
type User_R struct {
	T_uuid        string //
	T_power       string // 权限ID
	T_name        string //
	T_user        string //
	T_dept        int    // 部门
	T_dept_name   string // 部门
	T_post        int    // 岗位
	T_post_name   string // 岗位
	T_dept_leader int    //部门负责人

	T_sex                 int
	T_id_card             string
	T_nation              string
	T_school              string
	T_major               string
	T_education           string
	T_phone               string
	T_marry               int
	T_spouse_name         string
	T_spouse_phone        string
	T_entry_time          string
	T_positive_time       string
	T_entry_type          string
	T_contract_start_time string
	T_contract_end_time   string
	T_expire              int
	T_remark              string
	T_State               int
}

var User_r *User

func (t *User) TableName() string {
	return "user" // 数据库名称   // ************** 替换 FormulaList **************
}

func init() {
	//注册模型
	orm.RegisterModel(new(User))

}
func UserToUser_R(r User) (m User_R) {
	m.T_uuid = r.T_uuid
	m.T_power = r.T_power
	m.T_name = r.T_name
	m.T_user = r.T_user
	m.T_dept = r.T_dept
	m.T_dept_name = Read_Dept_Get(r.T_dept)
	m.T_post = r.T_post
	m.T_post_name = Read_Dept_Get(r.T_post)
	m.T_dept_leader = r.T_dept_leader

	m.T_sex = r.T_sex
	m.T_id_card = r.T_id_card
	m.T_nation = r.T_nation
	m.T_school = r.T_school
	m.T_major = r.T_major
	m.T_education = r.T_education
	m.T_phone = r.T_phone
	m.T_marry = r.T_marry
	m.T_spouse_name = r.T_spouse_name
	m.T_spouse_phone = r.T_spouse_phone
	m.T_entry_time = r.T_entry_time
	m.T_positive_time = r.T_positive_time
	m.T_entry_type = r.T_entry_type
	m.T_contract_start_time = r.T_contract_start_time
	m.T_contract_end_time = r.T_contract_end_time
	m.T_expire = r.T_expire
	m.T_remark = r.T_remark
	m.T_State = r.T_State

	return
}

func UserToUser_Libs(r User) (m userlibs.User) {
	m.T_uuid = r.T_uuid
	m.T_power = r.T_power
	m.T_name = r.T_name
	m.T_user = r.T_user
	m.T_dept = r.T_dept
	m.T_dept_name = Read_Dept_Get(r.T_dept)
	m.T_post = r.T_post
	m.T_post_name = Read_Dept_Get(r.T_post)
	m.T_dept_leader = r.T_dept_leader
	m.T_State = r.T_State
	return
}

// 验证登录
func Read_User_verification(T_user string, T_pass string) (error, User) {
	o := orm.NewOrm()
	r := User{T_user: T_user, T_pass: T_pass, T_State: 1}
	err := o.Read(&r, "T_user", "T_pass", "T_State") // o.Read(&r,"Tokey") 如果不是 主键 就得指定字段名
	if err != nil {
		logs.Error(lib.FuncName(), err)
	}
	return err, r
}

// 添加
func Add_User(r User) (id int64, err error) {
	o := orm.NewOrm()

	//生成编号
	rand_x := 0
	for true {
		r.T_uuid = lib.GetRandstring(32, "", int64(rand_x)) // 1,336,336
		err = o.Read(&r, "T_uuid")                          // o.Read(&r,"Tokey") 如果不是 主键 就得指定字段名
		if err != nil {
			break
		}
		rand_x += 1
	}
	// 查看是否重复
	err = o.Read(&r, "T_user") // o.Read(&r,"Tokey") 如果不是 主键 就得指定字段名
	if err == nil {
		return 0, errors.New("T_user !!!!")
	}

	r.T_State = 1
	id, err = o.Insert(&r)
	if err != nil {
		logs.Error(lib.FuncName(), err)
	}
	return id, err
}

// 获取 ById
func Read_User_ByT_uuid(T_uuid string) (r User, err error) {
	o := orm.NewOrm()
	qs := o.QueryTable(new(User))
	err = qs.Filter("T_uuid", T_uuid).One(&r)
	if err != nil {
		logs.Error(lib.FuncName(), err)
	}
	return
}

// 修改
func Update_User(m User, cols ...string) error {
	o := orm.NewOrm()
	num, err := o.Update(&m, cols...)
	if err != nil {
		logs.Error(lib.FuncName(), err)
		return err
	}
	fmt.Println("Number of records updated in database:", num)
	return nil
}

// 删除
func Delete_User(v User) error {
	o := orm.NewOrm()
	v.T_State = 0
	_, err := o.Update(&v, "T_State")
	if err != nil {
		logs.Error(lib.FuncName(), err)
	}
	return err
}

// 获取列表
func Read_User_List(T_name string, T_power string, T_dept, T_dept_leader, T_State, page, page_z int) (r_ []User_R, cnt int64) {
	o := orm.NewOrm()
	// 也可以直接使用 Model 结构体作为表名
	qs := o.QueryTable(new(User))
	var offset int64
	if page <= 1 {
		offset = 0
	} else {
		offset = int64((page - 1) * page_z)
	}

	// 过滤
	cond := orm.NewCondition()
	cond1 := cond.And("T_State__gt", 0) //.And("T_State", 1) .AndNot("status__in", 1).Or("profile__age__gt", 2000)

	if len(T_name) > 0 {
		//cond1.AndCond(cond.And("T_class", class))
		cond1 = cond1.AndCond(cond.Or("T_name__icontains", T_name).Or("T_user__icontains", T_name))
	}
	if len(T_power) > 0 {
		cond1 = cond1.And("T_power", T_power)
	}
	if T_dept > 0 {
		cond1 = cond1.And("T_dept", T_dept)
	}

	if T_dept_leader > 0 {
		cond1 = cond1.And("T_dept_leader", 1)
	}

	if T_State > 0 {
		cond1 = cond1.And("T_State", T_State)
	}

	// 查询
	var r []User
	_, err := qs.Limit(page_z, offset).SetCond((*orm2.Condition)(cond1)).OrderBy("-Id").All(&r)

	if err != nil {
		logs.Error(lib.FuncName(), err)
		return
	}

	cnt, err = qs.SetCond((*orm2.Condition)(cond1)).Count()
	if err != nil {
		logs.Error(lib.FuncName(), err)
		return
	}

	for _, v := range r {
		r_ = append(r_, UserToUser_R(v))
	}

	return r_, cnt
}
func Read_User_List_ByT_uuid(T_name string, T_uuid_list []string, page, page_z int) (r_ []User_R, cnt int64) {
	o := orm.NewOrm()
	// 也可以直接使用 Model 结构体作为表名
	qs := o.QueryTable(new(User))
	var offset int64
	if page <= 1 {
		offset = 0
	} else {
		offset = int64((page - 1) * page_z)
	}

	// 过滤
	cond := orm.NewCondition()
	cond1 := cond.And("T_State__gt", 0) //.And("T_State", 1) .AndNot("status__in", 1).Or("profile__age__gt", 2000)

	if len(T_name) > 0 {
		//cond1.AndCond(cond.And("T_class", class))
		cond1 = cond1.AndCond(cond.Or("T_name__icontains", T_name).Or("T_user__icontains", T_name))
	}
	if len(T_uuid_list) > 0 {
		cond1 = cond1.And("T_uuid__in", T_uuid_list)
	}

	// 查询
	var r []User
	_, err := qs.Limit(page_z, offset).SetCond((*orm2.Condition)(cond1)).OrderBy("-Id").All(&r)

	if err != nil {
		logs.Error(lib.FuncName(), err)
		return
	}

	cnt, err = qs.SetCond((*orm2.Condition)(cond1)).Count()
	if err != nil {
		logs.Error(lib.FuncName(), err)
		return
	}

	for _, v := range r {
		r_ = append(r_, UserToUser_R(v))
	}

	return r_, cnt
}

// 获取列表
func Read_User_List_All() (UserList []User_R) {

	o := orm.NewOrm()
	// 也可以直接使用 Model 结构体作为表名

	qs := o.QueryTable(new(User))
	var maps []User

	_, err := qs.All(&maps)
	if err != nil {
		logs.Error(lib.FuncName(), err)
		return UserList
	}
	for _, v := range maps {
		UserList = append(UserList, UserToUser_R(v))
	}

	return UserList
}
